/* Remember the IP for a label */
static void bas_add_label(fawk_ctx_t *ctx, char *lab, double lineno)
{
	char tmp[128];

	if (lab == NULL) sprintf(lab = tmp, "%ld", (long)lineno);
	if (htpp_has(ctx->compiler.labels, lab)) libfawk_error(ctx, "Ignoring duplicate label");
	else htpp_set(ctx->compiler.labels, fawk_strdup(ctx, lab), (void *)(long)ctx->code.used+1);
}

static void bas_add_jump(fawk_ctx_t *ctx, char *lab, double lineno)
{
	char tmp[128];
	void *ip;

	if (lab == NULL) sprintf(lab = tmp, "%ld", (long)lineno);

	fawkc_addi(ctx, FAWKI_JMP);
	ip = (void *)ctx->code.used; htpp_set(ctx->compiler.lablink, ip, (void *)fawk_strdup(ctx, lab));
	fawkc_addnum(ctx, 7771);
}

static int bas_init_labels(fawk_ctx_t *ctx)
{
	ctx->compiler.labels = fawk_malloc(ctx, sizeof(htpp_t));
	if (ctx->compiler.labels == NULL) return -1;
	ctx->compiler.lablink = fawk_malloc(ctx, sizeof(htpp_t));
	if (ctx->compiler.lablink == NULL) {fawk_free(ctx, ctx->compiler.labels); ctx->compiler.labels = NULL; return -1; }
	htpp_init(ctx->compiler.labels, (unsigned int (*)(const void *))strhash_case, (int (*)(const void *, const void *))strkeyeq_case);
	htpp_init(ctx->compiler.lablink, ptrhash, ptrkeyeq);
	return 0;
}

static int bas_uninit_labels(fawk_ctx_t *ctx)
{
	htpp_entry_t *e;

	for(e = htpp_first(ctx->compiler.lablink); e != NULL; e = htpp_next(ctx->compiler.lablink, e)) {
		void *addr = htpp_get(ctx->compiler.labels, e->value);
		if (addr == NULL) { libfawk_error(ctx, "Undefined goto label:"); libfawk_error(ctx, e->value); return -1; }
		ctx->code.code[(long)e->key].data.num = (long)addr-1;
		fawk_free(ctx, e->value);
	}

	for(e = htpp_first(ctx->compiler.labels); e != NULL; e = htpp_next(ctx->compiler.labels, e))
		free(e->key);
	htpp_uninit(ctx->compiler.labels);
	htpp_uninit(ctx->compiler.lablink);
	fawk_free(ctx, ctx->compiler.labels); ctx->compiler.labels = NULL;
	fawk_free(ctx, ctx->compiler.lablink); ctx->compiler.lablink = NULL;
	return 0;
}
